"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

import os, glob
from parseAwardXML import *
from downloadPUB import *
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from settings import *

engine = None
Session = None
session = None

def getDBConnectorString():
  mysql_settings = SETTINGS["MYSQL_SETTINGS"]
  return "mysql://%s:%s@%s/%s" % (mysql_settings["USER"], mysql_settings["PASSWORD"],\
                              mysql_settings["HOST"], mysql_settings["DB_NAME"])

def prepareDBConnection():
  global engine, Session,session
  print getDBConnectorString()
  engine = create_engine(getDBConnectorString())
  Session = sessionmaker(bind=engine)
  session = Session()

def getFiles(folderPath):
  print folderPath
  files = glob.glob(folderPath+SETTINGS["XML_MATCH_PATTERN"])
#  print files
  return files


def updateAllInDB(path):
  award_updated_count = 0
  full_path = os.path.abspath(path)
  print full_path
  for root, folders, _ in os.walk(full_path):
    for foldername  in folders:
      for file_name in getFiles(os.path.join(root, foldername)):
        addToDB(file_name)
        award_updated_count += 1
        print "Awards updated: ", award_updated_count

  return 0;

def addToDB(filePath):
  print filePath
  global session
  obj = Award(filePath)
  obj.createTableObj()
 # obj.showData()
  obj.addToSession(session)
  session.commit()


def addToSession(obj):
  global session
  session.merge(obj.table)
  session.merge(obj.program_element)
  for i in obj.investigators:
    session.merge(i)
  for i in obj.program_reference:
    session.merge(i)
  print "All classes added."

def updatePubData(pubfiles=None):
  global session
  obj = DownloadPUBData()
  obj.pubfiles = pubfiles
  obj.putAllInDB(session)
  session.commit()
  print "All pub data pushed in DB."



if __name__ == "__main__":
  import argparse
  parser = argparse.ArgumentParser()
  parser.add_argument("-t", "--downloadtype", help="n for NSF data, p for Publication data") # n for NSF data, p for Publication data
  args = parser.parse_args()
  if args.downloadtype != "n" and args.downloadtype != "p":
    parser.print_help()
    exit(-1)
  prepareDBConnection()
  if args.downloadtype == "n":
    updateAllInDB(SETTINGS["XML_DATA"])
    print "NSF Data Updated in DB."
  elif args.downloadtype == "p":
    updatePubData()
    print "Publication Data downloaded and updated in DB"
  print "Done"
  exit(0)
