"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

from bs4 import BeautifulSoup
import urllib2
import cgi
from settings import *
import datetime
import re


class DownloadNSF:
 
  def __init__(self):
    print SETTINGS
    self.url = SETTINGS['NSF_BASE_URL']+"download.jsp"
    self.downloadDateTime = datetime.datetime.strptime(SETTINGS['LAST_UPDATED_DOWNLOAD'], "%m/%d/%Y %H:%M:%S")
    self.page = urllib2.urlopen(self.url).read()
    self.isNewDownload = False
    self.updatedDateTimeString = SETTINGS['LAST_UPDATED_DOWNLOAD']
    self.soup = BeautifulSoup(self.page)
    self.soup.prettify()

    if not self.isUpdated():
      print "Updating Zip Files. Will take some time: "
      self.start_download()
    else:
      print "Updated to the latest version."

  def getUpdatedDateTime(self):
    updateInfoDiv = self.soup.find(id="downloadText")
    updateInfo = updateInfoDiv.text
    match = re.search(r'\d{2}/\d{2}/\d{4}\ \d{2}:\d{2}:\d{2}', updateInfo)
    dateString = match.group(0)
    updateDateTime = datetime.datetime.strptime(dateString, "%m/%d/%Y %H:%M:%S")
    self.updatedDateTimeString = dateString
    return updateDateTime

  def isUpdated(self):
    if self.downloadDateTime < self.getUpdatedDateTime():
      return False
    return True

  def start_download(self):
    for div in self.soup.find_all("div", "downloadcontent"):
        print div.text
        for link in div.find_all("a"):
            fileLink = SETTINGS['NSF_BASE_URL']+link.get("href")
            u = urllib2.urlopen(fileLink)
            _, params = cgi.parse_header(u.info()['Content-Disposition'])
            fileName = params['filename']
            print "Downloading {0} from url: {1}".format(fileName, fileLink)
            zipFilePath = SETTINGS['ZIP_DATA']+fileName
            localFile = open(zipFilePath, 'w')
            localFile.write(u.read())
            localFile.close()
            print "File {0} downloaded.".format(zipFilePath)
    print "New Download Success."
    print "Updating setting."
    update({"LAST_UPDATED_DOWNLOAD" : self.updatedDateTimeString})
    self.isNewDownload = True
    return 0


if __name__ == "__main__":
  print "Invoked from shell. Will check for download status."
  obj = DownloadNSF()


