"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

from bs4 import BeautifulSoup
from mappingClasses import AwardPubDataDB
import urllib2
from settings import *
import datetime
import os, re
import csv

class DownloadPUBData:
  def __init__(self):
    self.lookup_url = SETTINGS['PUBDATA_BASE_URL']+SETTINGS['PUBDATA_DOWNLOAD_URI']
    self.pubfiles = []
    self.page = urllib2.urlopen(self.lookup_url).read()
    self.soup = BeautifulSoup(self.page)
    # self.start_download()

  def start_download(self):
    for link in self.soup.find_all("a", "exportCSV"):
      print link.get("href")
      fileLink = SETTINGS['PUBDATA_BASE_URL']+link.get("href")
      u = urllib2.urlopen(fileLink)
      pubFilePath = SETTINGS['PUB_DATA']+os.path.basename(fileLink)
      with open(pubFilePath, "wb") as local_file:
        print "Downloading {0} from url: {1}".format(pubFilePath, fileLink)
        local_file.write(u.read())
        print "File {0} downloaded.".format(pubFilePath)
      self.pubfiles.append(pubFilePath)
    print "Pub data download success."
    return 0

  def getData(self, filepath):
    csv_file = open(filepath, "r")
    csv_dict_reader = csv.DictReader(x.replace('\0', '') for x in csv_file)
    csv_data = []
    for row in csv_dict_reader:
      if row['Agency'] != 'NSF':
        continue
      csv_data.append(self.getRowData(row))
    return csv_data

  def putInDB(self, filepath, session):
      csv_data = self.getData(filepath)
      i = 0
      for data in csv_data:
        obj = AwardPubDataDB(data)
        session.merge(obj)
        i += 1
        print "Object %s added to DB: %s" %(i, obj)
      session.commit()
      print "Total %s objects added to DB." %(i)
      return i


  def putAllInDB(self, session, pubfiles=None):
    if pubfiles is None:
      pubfiles = self.pubfiles
      i = 0
    for filepath in pubfiles:
      print "Pushing contents of %s in DB." % (filepath)
      i += self.putInDB(filepath, session)
    print "Total %s objects added to DB." %(i)
    return 0

  def getRowData(self, row):
    dict_data = {}
    dict_data['AwardID'] = self.sanitizeData(row['Federal Award ID Number'])
    dict_data['PubResult'] = row['Publications Produced as a Result of this Research']
    dict_data['PubConf'] = row['Publications Produced as Conference Proceedings']
    dict_data['ProjectOutcomesReport'] = row['ProjectOutcomesReport']
    dict_data['GrantType'] = row['Transaction Type']
    dict_data['DUNSID'] = self.sanitizeData(row['DUNS ID'])
    dict_data['ParentDUNSID'] = self.sanitizeData(row['Parent DUNS ID'])
    dict_data['AwardDetails'] = row['Award Title or Description']
    dict_data['AwardAbstract'] = row['Abstract at Time of Award']
    dict_data['AwardeeOrg'] = row['Awardee']
    dict_data['Businessname'] = row['Doing Business As Name']
    dict_data['EstAmount'] = self.sanitizeData(row['Estimated Total Award Amount'])
    dict_data['UsedAmount'] = self.sanitizeData(row['Funds Obligated to Date'])
    dict_data['CFDANumber'] = self.sanitizeData(row['CFDA Number'])
    dict_data['Program'] = row['Program']
    dict_data['AwardeeCity'] = row['Awardee City']
    dict_data['AwardeeState'] = row['Awardee State']
    dict_data['AwardeeCounty'] = row['Awardee County']
    dict_data['AwardeeZip'] = row['Awardee ZIP']
    dict_data['AwardeeCountry'] = row['Awardee Country']
    dict_data['PrimaryOrg'] = row['Primary Organization Name']
    dict_data['PrimaryCity'] = row['Primary City']
    dict_data['PrimaryState'] = row['Primary State']
    dict_data['PrimaryCounty'] = row['Primary County']
    dict_data['PrimaryZip'] = row['Primary ZIP']
    dict_data['PrimaryCountry'] = row['Primary Country']
    return dict_data

  def sanitizeData(self, amt_str):
    return re.sub('[\"=$,]', '', amt_str)


if __name__ == "__main__":
  print "Invoked from shell. Will check for download status."
  obj = DownloadPUBData()
  obj.start_download()
