"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

from sqlalchemy import Column, Integer, String, Date, Text, Numeric
from sqlalchemy.ext.declarative import declarative_base

Base = declarative_base()
class AwardDB(Base):
  __tablename__ = 'nsf_grants'
  AwardID = Column(String(10), primary_key=True)
  AwardTitle = Column(String(300))
  AwardEffectiveDate = Column(Date)
  AwardExpirationDate = Column(Date)
  AwardAmount = Column(Integer)
  AwardInstrument = Column(String(50))
  NSFOrganizationCode = Column(String(10))
  NSFDirectorate = Column(String(200))
  NSFDivision = Column(String(200))
  ProgramOfficer = Column(String(50))
  AbstractNarration = Column(Text)
  MinAmdLetterDate = Column(Date)
  MaxAmdLetterDate = Column(Date)
  ARRAAmount = Column(Integer)
  PrincipalInvestigator = Column(String(100))
  PIEmailAddress = Column(String(100))
  CoPINames = Column(String(200))
  Institution = Column(String(200))
  InstitutionStreet = Column(String(200))
  InstitutionCity = Column(String(20))
  InstitutionState = Column(String(50))
  InstitutionCountry = Column(String(50))
  InstitutionZip = Column(Integer)
  InstitutionPhone = Column(String(20))
  ProgramElementCodes = Column(String(50))
  ProgramReferenceCodes = Column(String(50))

  def __init__(self):
    print "AwardDB object created."
    None
#
#  def __init__(self, data_dict):
#    for k in data_dict:
#      self.__dict__[k] = data_dict[k]

  def __repr__(self):
    return "<Award('%s', '%s')>" % (self.AwardID, self.AwardTitle)

  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])

class InvestigatorDB(Base):
  __tablename__ = 'investigator_data'
  AwardID = Column(String(10), primary_key=True)
  RoleCode = Column(Integer, primary_key=True)
  FirstName = Column(String(50))
  LastName = Column(String(50))
  EmailAddress = Column(String(100))
  StartDate = Column(Date)
  EndDate = Column(Date)
  def __init__(self):
    print "InvestigatorDB object created."
    None
  def __repr__(self):
    return "<Investigator('%s', '%s, %s, %s')>" % (self.AwardID, self.RoleCode, self.FirstName, self.LastName)
  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])

class ProgramElementDB(Base):
  __tablename__ = "program_elements"
  Code = Column(Integer, primary_key=True)
  Text = Column(String(100))
  def __init__(self):
    print "ProgramELementDB object created."
    None
  def __repr__(self):
    return "<ProgramElement('%s', '%s')>" % (self.Code, self.Text)
  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])

class ProgramReferenceDB(Base):
  __tablename__ = "program_reference"
  Code = Column(Integer, primary_key=True)
  Text = Column(String(100))
  def __init__(self):
    print "ProgramReferenceDB object created."
    None
  def __repr__(self):
    return "<ProgramReferene('%s', '%s')>" % (self.Code, self.Text)
  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])

class AwardPubDataDB(Base):
  __tablename__ = 'award_pub_data'
  AwardID = Column(String(10), primary_key=True)
  PubResult = Column(Text())
  PubConf = Column(Text())
  ProjectOutcomesReport = Column(Text())
  GrantType = Column(String(30))
  DUNSID = Column(String(10))
  ParentDUNSID = Column(String(10))
  AwardDetails = Column(Text())
  AwardAbstract = Column(Text())
  AwardeeOrg = Column(String(300))
  Businessname = Column(String(300))
  EstAmount = Column(Integer())
  UsedAmount = Column(Numeric())
  CFDANumber = Column(String(10))
  Program = Column(String(300))
  AwardeeCity = Column(String(100))
  AwardeeState = Column(String(3))
  AwardeeCounty = Column(String(30))
  AwardeeZip = Column(String(20))
  AwardeeCountry = Column(String(10))
  PrimaryOrg = Column(String(300))
  PrimaryCity = Column(String(100))
  PrimaryState = Column(String(3))
  PrimaryCounty = Column(String(30))
  PrimaryZip = Column(String(20))
  PrimaryCountry = Column(String(10))

  def __init__(self, data_dict=None):
    if data_dict is not None:
      for k in data_dict:
        self.__dict__[k] = data_dict[k]
    print "AwardPubDataDB object created."

  def __repr__(self):
    return "<AwardPubDataDB('%s', '%s')>" % (self.AwardID, self.AwardDetails)

  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])


class PubRecordsDB(Base):
  __tablename__ = 'pub_records'
  PUB_TYPES = {"RESULT": 0, "CONF": 1}
  AwardID = Column(String(10), primary_key=True)
  PubType = Column(Integer(), primary_key=True)
  PubOrder = Column(Integer(), primary_key=True)
  PubString = Column(Text())
  PubTitle = Column(Text())
  PubAuthors = Column(Text())
  PubData = Column(Text())

  def __init__(self, data_dict=None):
    if data_dict is not None:
      for k in data_dict:
        self.__dict__[k] = data_dict[k]
    print "PubRecordsDB object created."

  def __repr__(self):
    return "<PubRecordsDB('%s', '%s')>" % (self.AwardID, self.PubString)

  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])

class CollaborativeGrantsDB(Base):
  __tablename__ = 'collaborative_grants'
  CollabID = Column(Integer(), primary_key=True)
  AwardID = Column(String(10), primary_key=True)
  AwardTitle = Column(String(300))

  def __init__(self, data_dict=None):
    if data_dict is not None:
      for k in data_dict:
        self.__dict__[k] = data_dict[k]
    print "CollaborativeGrantsDB object created."

  def __repr__(self):
    return "<CollaborativeGrantsDB('%s', '%s', '%s')>" % (self.CollabID, self.AwardID, self.AwardTitle)

  def showData(self):
    for k in self.__dict__:
      print "%s : %s" % (k, self.__dict__[k])
