"""
Project: NSF Award Data
Author: Shubhanshu Mishra <smishra8@illinois.edy>
NSF Grant ID: 1348742
NSF Grant Title: Collaborative Research: STEM Workforce Training: A Quasi-Experimental Approach Using the Effects of Research Funding
Investigator(s): Vetle Torvik vtorvik@illinois.edu (Principal Investigator)
Date: 12/9/2013
Copyright 2013, University of Illinois at Urbana-Champaign
"""

from dbInterface import *
from mappingClasses import AwardPubDataDB, PubRecordsDB
from sqlalchemy import or_

def parseResults():
  for AwardID, PubResult, PubConf in session.query(AwardPubDataDB.AwardID, AwardPubDataDB.PubResult, AwardPubDataDB.PubConf).\
      filter(or_(AwardPubDataDB.PubResult != '', AwardPubDataDB.PubConf != '')).\
      order_by(AwardPubDataDB.AwardID):
    print "%s\t%s\t%s" %(AwardID, PubResult, PubConf)
    total_pubs = 0
    total_pubs += setupPubRecord(AwardID, PubResult.split("~, "), PubRecordsDB.PUB_TYPES["RESULT"])
    total_pubs += setupPubRecord(AwardID, PubConf.split("~, "), PubRecordsDB.PUB_TYPES["CONF"])
    if total_pubs > 0:
      session.commit()
      print "Award ID %s details written to DB. Pub count: %s." %(AwardID, total_pubs)

def setupPubRecord(award_id, pub_list, pub_type):
  if len(pub_list) == 0:
    return 0
  print "[%s] => Pub List: %s" %(award_id, pub_list)
  pub_added = 0
  data_list = ["Authors", "Title", "Data"]
  dummy_dict = {}
  for pub in pub_list:
    if pub == '':
      break
    pub_obj = PubRecordsDB()
    pub_obj.AwardID = award_id
    pub_obj.PubType = pub_type
    pub_obj.PubOrder = pub_added + 1
    pub_obj.PubString = pub
    pub_data = pub.split("~")
    dummy_dict = {"Authors": "", "Title": "", "Data": ""}
    for i in range(len(pub_data)):
      if i == 2:
        dummy_dict[data_list[i]] = "~".join(pub_data[2:])
        break
      else:
        dummy_dict[data_list[i]] += pub_data[i]
    pub_obj.PubTitle = dummy_dict["Title"]
    pub_obj.PubAuthors = dummy_dict["Authors"]
    pub_obj.PubData = dummy_dict["Data"]
    session.merge(pub_obj)
    pub_added += 1
    print "Object Added: ", pub_obj.showData()
  return pub_added

if __name__ == "__main__":
  parseResults()
